# -*- coding: utf-8 -*-
import logging
import threading

try:
    import sentry_sdk as sentry
except ImportError:
    logging.getLogger(__name__).info('sentry_sdk not available')
else:
    logging.getLogger(__name__).info('sentry enabled')
    from odoo.addons.saas_worker.config import saas_config
    from odoo.addons.saas_worker.const import TEST
    from odoo.release import version
    from odoo.tools import config
    from collections import defaultdict
    # We don't want sentry to send INFO/DEBUG/WARNING exception, only errors
    sentry.set_level("error")

    SENTRY_DSN_KEY = saas_config.get('trial', {}).get('sentry_dsn_key')
    if not SENTRY_DSN_KEY:
        logging.getLogger(__name__).error('sentry dsn key not found!')

    IGNORED_EXCEPTIONS = [
        # Exception ignored by sentry
        'HTTPError',
        'PoolError',
        'RedirectWarning',
        'MailDeliveryException',
        'SerializationFailure',
        'InvalidDatabaseException',
        'Forbidden',
        'UserError',
        'ValidationError',
        'AccessError',
        'AccessDenied',
        'SignupError',
        'NumericValueOutOfRange',
        'No possible route found for incoming message from',
        'GSTIN is required under GST Treatment',
        'Signup: no name or partner given for new user',
        'Signup: no login given for new user',
        'DecompressionBombWarning',
        'DecompressionBombError',
        'IntegrityError',
        'OperationalError',
        'No transaction found matching reference',
        'UnicodeEncodeError',
        'InvalidVapidError',
        'CPU time limit exceeded',
        'InvalidTextRepresentation',
        'ReadTimeout',
        'ReadTimeoutError',
        'TimeoutError',
        'InvalidToken',
    ]

    IGNORED_LOGS = [
        # Logs ignored by sentry
        'bad query:',
        'Missing model',
        'Failed to insert ir_model_data',
        'saas_trial/create_invitation: login check failed for mails',
        'saas_trial/create_invitation: secret signature checking failed for mails',
        'Error when reading the pdf',
        'Error when fetching LinkedIn stats',
        'A batch of leads could not be enriched',
        'Non-stored field %s cannot be searched.',
        'Wrong state value',
        'An error occurred while fetching the comment',
        'Some modules have inconsistent states, some dependencies may be missing',
        'SignupError',
        'You must either provide a sender address explicitly',
        'Database %s: module(s) %s not found',
        'Error while importing module',
        'Ebay: cannot fetch OAuth token without credentials.',
        'Couldn\'t fetch OAuth token from Ebay',
        'Auth token is hard expired, User needs to generate a new token for this application',
        'Exception while sending traceback by email',
        'Failed to load registry',
    ]

    IGNORED_LOGGERS = [
        # Loggers ignored by sentry
        'odoo.addons.base_automation.models.base_automation',
        'longpolling',
    ]

    IGNORED_TERMS = [
        # Exception containing these terms will be ignored
        'x_studio',
    ]

    events_per_db = defaultdict(set)

    def is_exception_ignored(exc_type, exc_value):
        for ignored_exc in IGNORED_EXCEPTIONS:
            if exc_type.__name__ == ignored_exc or ignored_exc in str(exc_value):
                return True
        return False

    # This function will be called each time sentry tries to send an error.
    # It is used to filter what we send and what we don't send.
    # If we return None, the sending will be canceled
    # We can also modify the event before sending it
    def filter_data(event, hint):
        # Add the dbname tag so that it can be seen from the Sentry interface
        dbname = getattr(threading.current_thread(), 'dbname', None)
        event.setdefault('tags', {})
        event['tags']['dbname'] = dbname
        error = None
        if 'exc_info' in hint:
            exc_type, exc_value, _tb = hint['exc_info']
            error = "%s %s" % (exc_type, exc_value)
            if is_exception_ignored(exc_type, exc_value) or getattr(exc_value, 'sentry_ignored', False):
                return None
            elif any(term in str(exc_value) for term in IGNORED_TERMS):
                return None
        elif 'log_record' in hint:
            error = hint['log_record'].msg
            message = hint['log_record'].message
            if any([log in error for log in IGNORED_LOGS]):
                return None
            elif any(term in str(message) for term in IGNORED_TERMS):
                return None
        if 'logger' in event:
            if event['logger'] in IGNORED_LOGGERS:
                return None
        db_errors = events_per_db[dbname]
        if error in db_errors:
            return None
        db_errors.add(error)
        return event

    if SENTRY_DSN_KEY and not TEST and not config["stop_after_init"]:
        sentry.init(
            dsn=SENTRY_DSN_KEY,
            traces_sample_rate=1.0,  # Set to 1.0 to capture 100% of transactions
            max_breadcrumbs=100,
            before_send=filter_data,
            release=version)
